﻿using System.Linq;
using System.Web.Mvc;
using JQueryGridWeb.Models;

namespace JQueryGridWeb.Controllers
{
    [HandleError]
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            ViewData["Message"] = "Welcome to ASP.NET MVC!";

            return View();
        }

        public JsonResult GridData(string sidx, string sord, int page, int rows)
        {
            const int totalPages = 1;
            const int totalRecords = 3;

            var jsonData = new
            {
                total = totalPages,
                page,
                records = totalRecords,
                rows = new[]{
                    new {id = 1, cell = new[] {"1", "-7", "Is this a good question?"}},
                    new {id = 2, cell = new[] {"2", "15", "Is this a blatant ripoff?"}},
                    new {id = 3, cell = new[] {"3", "23", "Why is the sky blue?"}}
                }
            };
            return Json(jsonData);
        }

        public JsonResult LinqGridData(string sidx, string sord, int page, int rows)
        {
            //var context = new HaackOverflowDataContext();
            var context = new HaackOverflowEntities ();
            var questionDatas = (from question in context.Questions
                                 select new { question.Id, question.Votes, question.Title }).ToList ();

            var jsonData = new
            {
                total = 1, //todo: calculate
                page,
                records = context.Questions.Count(),
                rows = (from question in questionDatas
                        select new
                        {
                            id = question.Id,
                            cell = new[] { question.Id.ToString(), question.Votes.ToString(), question.Title }
                        }).ToList()
            };
            return Json(jsonData);
        }

        public JsonResult DynamicGridData(string sidx, string sord, int page, int rows)
        {
            //var context = new HaackOverflowDataContext();
            var context = new HaackOverflowEntities ();
            var pageIndex = page - 1;
            var pageSize = rows;
            var totalRecords = context.Questions.Count();
            //int totalPages = (int)Math.Ceiling((float)totalRecords / (float)pageSize);
            var totalPages = (totalRecords + pageSize - 1) / pageSize;

            var questions = context.Questions.OrderBy("it." + sidx + " " + sord).Skip(pageIndex * pageSize).Take(pageSize);

            // to be able to use ToString() below which is NOT exist in the LINQ to Entity
            // we get the properties which we need and convert results to List
            var questionDatas = (from question in questions
                                 select new { question.Id, question.Votes, question.Title }).ToList ();

            var jsonData = new
            {
                total = totalPages,
                page,
                records = totalRecords,
                rows = (from question in questionDatas
                        select new
                        {
                            id = question.Id,
                            cell = new[] { question.Id.ToString(), question.Votes.ToString(), question.Title }
                        }).ToList()
            };
            return Json(jsonData);
        }

        public ActionResult About()
        {
            return View();
        }
    }
}
